/* Arduino Quadruped Robot: Walks, Skims & Grips
 https://www.instructables.com/Arduino-Quadruped-Robot-Walks-Skims-Grips/
    By JD_K
    July 2021

   Pinning of the PWM driver to the Arduino:
    GND to GND
    OE not used
    SCL of the driver to A5 of the arduino (nano)
    SDA to A4
    Vcc of the driver to 5V of the arduino
    V+ not used

  Pinning for the IR receiver for the remote:
  G to GND on Arduino
  R (middle) to 5V
  Y to pin 12

  Pinning for the LED
  anode (long leg) of LED to pin 13 of Arduino
  cathode of LED to 1k Ohm resistor
  resistor to GND of Arduino
*/

#include <IRremote.h>
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

#define SERVOMIN  150 // this is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX  600 // this is the 'maximum' pulse length count (out of 4096)
int delayTime = 3; // duration between increments of walking movements, affects the speed of walking and smoothness
int rowTime = 230; // duration for the stroke from skimming movement
int togglePosture = 3; // toggled to track the postures of the robot: skimming=0, or standing=1, or grip=2, or none of these=3

// Setting up the variables for the servos with the values for retracted legs*********************
int hip1 = 400; //motor 0 on the PWM motor driver; note: #3, 7 and 11 are skipped
int fem1 = 500; //motor 1
int tib1 = 500; // motor2
int hip2 = 270; // motor 4
int fem2 = 200; // motor 5
int tib2 = 230; // motor 6
int hip3 = 420; // motor 8
int fem3 = 525; // motor 9
int tib3 = 500; // motor 10
int hip4 = 270; // motor 12
int fem4 = 200; // motor 13
int tib4 = 200; // motor 14

// Setting up some variables for specific positions
int legLevel = 5; //Leg Level: used to raise, lower front legs when gripping, flexed up=5, highest postion with tibia horizontal=4, lowest postion with tibia horizontal= 0
int legDist = 0; //Leg Distance: used to increment front legs together and apart for gripping, legDist=0 at a open position for gripping, legDist=20 when both legs point anteriorly
int Rt; // will be used for hip4 when gripping
int Lt; // will be used for hip1 when gripping
int RtTogetherLimit = 180; // limit for gripping - from trial and error, hip4 as close to front center as possible without causing servo buzzing when too close
int LtTogetherLimit = 540; // limit for gripping - from trial and error, hip1 as close to front center as possible without causing servo buzzing when too close
int RtApartLimit = 220; // for gripping - maximum deviation away from center
int LtApartLimit = 450;

// *************** These are the codes for each button being used on the IR remote ************
#define P 0xFFA25D  // “POWER” for PAUSE function and retracting all legs
#define Fr 0xFF629D  // “Vol+” for Walk FORWARD
#define B 0xFFA857  // “Vol-“ for walk BACKWARD
#define L 0xFF22DD  // “Fast backward” for turn LEFT
#define R 0xFFC23D  // “Fast forward” for turn RIGHT
#define St 0xFF02FD  // "Pause" for STAND UP in neutral stance posture
#define Eq 0xFF9867  // “EQ” for the SKIM&GRIP posture – back legs will skim, front legs used to grip 
#define Lu 0xFF906F  // “Up” for FRONT LEGS UP when gripping
#define Ld 0xFFE01F  // “Down” for FRONT LEGS DOWN when gripping
#define La 0xFFB04F  // “ST/REPT” for the FRONT LEGS APART when gripping
#define lt 0xFF6897  // “0” for the FRONT LEGS TOGETHER when gripping
#define Sf 0xFF18E7  // “2” for SKIM FORWARD
#define Sb 0xFF4AB5  // “8” for SKIM BACKWARD
#define Sr 0xFF5AA5  // “6” for SKIM to the RIGHT
#define Sl 0xFF10EF  // “4” for SKIM to the LEFT
#define S 0xFF38C7  // “5” for SKIM Posture where all legs are out to the side


#define RECV_PIN  12 // receive pin on Arduino for the IR sensor 
IRrecv irrecv(RECV_PIN);
decode_results results;
unsigned long val;

#define LED_PIN 13 // will light up when on standby

// ********* Defining the Functions for the each button **************************************


void stand () {
  if (legLevel < 5) { // Note: if legLevel is less than 5, then the front legs are out, so first lower them in case something is being gripped:
    for (; tib1 > 100; tib1 = tib1 - 5) {
      if (tib4 < 600) tib4 = tib4 + 5;
      if (fem1 > 260) fem1 = fem1 - 3;
      if (fem4 < 340) fem4 = fem4 + 3;
      pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
      pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
      delay(10);
    }
    legLevel = 5;
  }
  for (; legDist > 0; legDist = legDist - 1) { // Now bring front legs apart, so that if anything was being gripped it should be released
    int Lt = map(legDist, 0, 20, LtApartLimit, LtTogetherLimit);
    int Rt = map (legDist, 0, 20, RtApartLimit, RtTogetherLimit);
    pwm.setPWM(0, 0, Lt); //hip1_545 anterior, 400 for neutral standing
    pwm.setPWM(12, 0, Rt); //hip4_150 anterior, 270 for neutral standing
  }
  // now in the next section all 4 legs should retract

  if (togglePosture != 1) {
    // Standing up: first retracting all the legs:
    hip1 = 400, fem1 = 500, tib1 = 500;
    hip2 = 270, fem2 = 200, tib2 = 230;
    hip3 = 420, fem3 = 550, tib3 = 500;
    hip4 = 270, fem4 = 200, tib4 = 200;
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
    pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
    delay (150);
    legLevel = 5; // because the legs are retracted
  }
  hip1 = 400, fem1 = 300, tib1 = 330; // now stand up
  hip2 = 270, fem2 = 400, tib2 = 400;
  hip3 = 420, fem3 = 300, tib3 = 330;
  hip4 = 270, fem4 = 400, tib4 = 375;

  pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
  pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
  pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
  pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
  delay(200);
  togglePosture = 1; // One (1) because it is now standing
  digitalWrite(LED_PIN, HIGH); // on standby
}

void forward() {
  if (legLevel < 5 || togglePosture != 1) { // Check if front legs are out (legLevel<5) or if not standing (toggle not equal to 1), if either of these are true then run the stand function:
    stand ();
  }

  // **********Walking: First shift towards leg4; hip1 at 400; hip3 at 420; tib4 at 400; tibia 2 at 400
  for (; tib2 < 500; tib2 = tib2 + 1) {
    if (fem2 < 425) fem2 = fem2 + 1;
    if (hip3 < 500) hip3 = hip3 + 1;
    if (fem4 < 410) fem4 = fem4 + 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(6, 0, tib2);
    pwm.setPWM(13, 0, fem4);
    delay (delayTime);
  }

  // ********** Walking: Second move leg2 forward ********************************
  for (; tib2 > 300; tib2 = tib2 - 2) {
    if (fem2 > 325) fem2 = fem2 - 1;
    if (fem1 > 290) fem1 = fem1 - 1;
    if (fem3 > 290) fem3 = fem3 - 1;
    if (tib1 > 290) tib1 = tib1 - 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(6, 0, tib2);
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(2, 0, tib1);
    delay (delayTime);
  }
  for (; hip2 < 420; hip2 += 1) {
    pwm.setPWM(4, 0, hip2);
    delay(delayTime);
  }
  for (; fem2 < 400; fem2 = fem2 + 1) {
    if (tib2 < 400) tib2 = tib2 + 1;
    if (fem1 < 300) fem1 = fem1 + 1;
    if (fem3 < 300) fem3 = fem3 + 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(6, 0, tib2);
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(9, 0, fem3);
    delay (delayTime);
  }

  // ************** Walking: Third movie leg 1 forward ************************************
  for (; fem1 < 350; fem1 = fem1 + 1) {
    if (tib1 < 425) tib1 = tib1 + 1;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(2, 0, tib1);
    delay(delayTime);
  }
  for (; hip1 < 475; hip1 = hip1 + 1) {
    if (tib1 < 350) tib1 = tib1 + 1;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(2, 0, tib1);
    delay(delayTime);
  }
  for (; tib1 > 250; tib1 = tib1 - 1) {
    pwm.setPWM(2, 0, tib1);
    delay(delayTime);
  }
  for (; fem1 > 275; fem1 = fem1 - 1) {
    if (tib1 > 175) tib1 = tib1 - 1;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(2, 0, tib1);
    delay(delayTime);
  }


  // ********* Walking: Forth shift towards leg1 then long step forward with leg3 ***************
  for (; hip2 > 275; hip2 = hip2 - 1) {
    if (tib1 < 330) tib1 = tib1 + 1;
    if (tib3 > 200) tib3 = tib3 - 1;
    if (hip4 < 400) hip4 = hip4 + 1;
    if (fem1 < 300) fem1 = fem1 + 1;
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(2, 0, tib1);
    pwm.setPWM(10, 0, tib3);
    pwm.setPWM(12, 0, hip4);
    pwm.setPWM(1, 0, fem1);
    delay(delayTime);
  }
  for (; fem3 < 375; fem3 = fem3 + 1) {
    if (tib3 < 375) tib3 = tib3 + 3;
    if (fem3 < 375) fem3 = fem3 + 1;
    if (hip2 > 250) hip2 = hip2 - 1;
    if (fem2 < 425) fem2 = fem2 + 1;
    if (hip4 < 425) hip4 = hip4 + 1;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(10, 0, tib3);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; hip3 > 325; hip3 = hip3 - 1) {
    pwm.setPWM(8, 0, hip3);
    delay(delayTime);
  }
  for (; fem3 > 300; fem3 = fem3 - 1) {
    if (tib3 > 330) tib3 = tib3 - 1;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(10, 0, tib3);
    delay(delayTime);
  }

  // *********** Walking: Fifth move leg4 anteriorly ****************************************
  for (; fem4 > 250; fem4 = fem4 - 1) {
    if (tib4 > 225) tib4 = tib4 - 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(14, 0, tib4);
    delay(delayTime);
  }
  for (; hip4 > 270; hip4 = hip4 - 1) {
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; tib4 < 300; tib4 = tib4 + 1) {
    pwm.setPWM(14, 0, tib4);
    delay(delayTime);
  }
  for (; fem4 < 400; fem4 = fem4 + 1) {
    if (fem2 > 400) fem2 = fem2 - 1;
    if (tib4 < 375) tib4 = tib4 + 1;
    if (hip1 > 400) hip1 = hip1 - 1;
    if (hip2 < 270) hip2 = hip2 + 1;
    if (hip3 < 420) hip3 = hip3 + 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(14, 0, tib4);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    delay(delayTime);
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  // ************ End Walking Forward ******************************************
}


void right() {
  // ****************Turning to the right **************************************************

  if (legLevel < 5 || togglePosture != 1) { // Check if front legs are out (<5) or if not standing (toggle not equal to 1)
    stand ();
  }

  // ********** Turning right from neutral standing: First move leg 4 to the right ******************
  for (; fem4 > 320; fem4 = fem4 - 8) { // lifting femur 4
    if (hip1 > 390) hip1 = hip1 - 1;
    if (hip2 > 260) hip2 = hip2 - 1;
    //    if (hip3 > 390) hip3 = hip3 - 1;
    if (hip3 > 410) hip3 = hip3 - 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    delay(delayTime);
  }
  for (; hip4 < 360; hip4 = hip4 + 9) { //moving hip4 to the side
    if (hip1 > 380) hip1 = hip1 - 1;
    if (hip2 > 250) hip2 = hip2 - 1;
    if (hip3 > 400) hip3 = hip3 - 1;
    pwm.setPWM(12, 0, hip4);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    delay(delayTime);
  }
  for (; fem4 < 400; fem4 = fem4 + 8) { //lowering leg4 a little more to take up more of the weight before lifting leg3 in the 2nd step
    if (hip1 > 370) hip1 = hip1 - 1;
    if (hip2 > 240) hip2 = hip2 - 1;
    if (hip3 > 390) hip3 = hip3 - 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    delay(delayTime);
  }

  // ****** Turning right from neutral standing: Second - moving leg3 to the right *****************
  for (; fem3 < 380; fem3 = fem3 + 8) { // Note fem4 in the last step was pushed down so the robot will not tip when lifting fem3 here
    if (hip1 > 360) hip1 = hip1 - 1;
    if (hip2 > 230) hip2 = hip2 - 1;
    if (hip4 > 350) hip4 = hip4 - 1;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; hip3 < 460; hip3 = hip3 + 9) {
    if (hip1 > 350) hip1 = hip1 - 1;
    if (hip2 > 220) hip2 = hip2 - 1;
    if (hip4 > 340) hip4 = hip4 - 1;
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; fem3 > 300; fem3 = fem3 - 8) {
    if (hip1 > 340) hip1 = hip1 - 1;
    if (hip2 > 210) hip2 = hip2 - 1;
    if (hip4 > 330) hip4 = hip4 - 1;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }

  // ******* Turning right from neutral standing: Third moving leg2 to the right ******************
  for (; fem2 > 320; fem2 = fem2 - 8) {
    if (hip1 > 330) hip1 = hip1 - 1;
    if (hip3 > 450) hip3 = hip3 - 1;
    if (hip4 > 320) hip4 = hip4 - 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }
  for (; hip2 < 300; hip2 = hip2 + 9) {
    if (hip1 > 320) hip1 = hip1 - 1;
    if (hip3 > 440) hip3 = hip3 - 1;
    if (hip4 > 310) hip4 = hip4 - 1;
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }
  for (; fem2 < 400; fem2 = fem2 + 8) {
    if (hip1 > 310) hip1 = hip1 - 1;
    if (hip3 > 430) hip3 = hip3 - 1;
    if (hip4 > 300) hip4 = hip4 - 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }

  // ********* Turning right from neutral standing: Forth move leg 1 to the right *******************
  for (; fem1 < 380; fem1 = fem1 + 8) {
    if (hip2 > 290) hip2 = hip2 - 1;
    if (hip3 > 420) hip3 = hip3 - 1;
    if (hip4 > 290) hip1 = hip1 - 1;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 00, hip3);
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }
  for (; hip1 < 400; hip1 = hip1 + 9) {
    if (hip2 > 280) hip2 = hip2 - 1;
    if (hip3 > 410) hip3 = hip3 - 1;
    if (hip4 > 280) hip1 = hip1 - 1;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }
  for (; fem1 > 300; fem1 = fem1 - 8) {
    if (hip2 > 270) hip2 = hip2 - 1;
    if (hip3 > 400) hip3 = hip3 - 1;
    if (hip4 > 270) hip4 = hip4 - 1;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  // *********** End turning to the right ****************************************
}


void left() {
  // ****************Turning to the left from neutral standing ********************************
  if (legLevel < 5 || togglePosture != 1) { // Check if front legs are out (legLevel<5) or if not standing (toggle not equal to 1)
    stand ();
  }

  // ********** Turning left from neutral standing: First move leg 4 to the left *****************
  for (; fem4 > 320; fem4 = fem4 - 8) { //raising leg4
    if (hip1 < 410) hip1 = hip1 + 1;
    if (fem1 > 234) fem1 = fem1 + 2;
    if (hip2 < 280) hip2 = hip2 + 1;
    if (fem3 > 284) fem3 = fem3 - 2;
    if (hip3 < 410) hip3 = hip3 + 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3);
    delay(delayTime);
  }
  for (; hip4 > 180; hip4 = hip4 - 9) { //moving hip4 forward
    if (hip1 < 420) hip1 = hip1 + 1;
    if (hip2 < 290) hip2 = hip2 + 1;
    if (hip3 < 420) hip3 = hip3 + 1;
    pwm.setPWM(12, 0, hip4);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    delay(delayTime);
  }
  for (; fem4 < 420; fem4 = fem4 + 10) {
    if (hip1 < 430) hip1 = hip1 + 1;
    if (fem1 < 250) fem1 = fem1 + 2;
    if (fem3 > 300) fem3 = fem3 + 2;
    if (hip2 < 300) hip2 = hip2 + 1;
    if (hip3 < 430) hip3 = hip3 + 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3);
    delay(delayTime);
  }
  // ******** Turning left from neutral standing: Second move leg 1 to the left *******************
  for (; fem1 < 350; fem1 = fem1 + 10) {
    if (hip2 < 310) hip2 = hip2 + 1;
    if (fem2 < 416) fem2 = fem2 + 2;
    if (hip3 < 440) hip3 = hip3 + 1;
    if (hip4 < 190) hip4 = hip4 + 1;
    if (fem4 < 416) fem4 = fem4 + 2;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2);
    pwm.setPWM(8, 00, hip3);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4);
    delay(delayTime);
  }
  for (; hip1 > 340; hip1 = hip1 - 9) {
    if (hip2 < 320) hip2 = hip2 + 1;
    if (hip3 < 450) hip3 = hip3 + 1;
    if (hip4 < 200) hip1 = hip1 + 1;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; fem1 > 250; fem1 = fem1 - 7) {
    if (hip2 < 330) hip2 = hip2 + 1;
    if (fem2 > 400) fem2 = fem2 - 2;
    if (hip3 < 460) hip3 = hip3 + 1;
    if (hip4 < 210) hip4 = hip4 + 1;
    if (fem4 > 400) fem4 = fem4 - 2;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4);
    delay(delayTime);
  }

  // ******* Turning left from neutral standing: Third moving leg2 to the left **********************
  for (; fem2 > 300; fem2 = fem2 - 10) {
    if (hip1 < 350) hip1 = hip1 + 1;
    if (fem1 > 234) fem1 = fem1 - 2;
    if (hip3 < 470) hip3 = hip3 + 1;
    if (fem3 > 284) fem3 = fem3 - 2;
    if (hip4 < 220) hip4 = hip4 + 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; hip2 > 240; hip2 = hip2 - 9) {
    if (hip1 < 360) hip1 = hip1 + 1;
    if (hip3 < 480) hip3 = hip3 + 1;
    if (hip4 < 230) hip4 = hip4 + 1;
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; fem2 < 400; fem2 = fem2 + 10) {
    if (hip1 < 370) hip1 = hip1 + 1;
    if (fem1 < 275) fem1 = fem1 + 3;
    if (fem3 < 300) fem3 = fem3 + 2;
    if (hip3 < 490) hip3 = hip3 + 1;
    if (hip4 < 240) hip4 = hip4 + 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }

  // ********* Turning left from neutral standing: Fourth moving leg3 to the left *******************
  for (; fem3 < 400; fem3 = fem3 + 10) {
    if (hip1 < 380) hip1 = hip1 + 1;
    if (hip2 < 250) hip2 = hip2 + 1;
    if (hip4 < 250) hip4 = hip4 + 1;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; hip3 > 400; hip3 = hip3 - 9) {
    if (hip1 < 390) hip1 = hip1 + 1;
    if (hip2 < 260) hip2 = hip2 + 1;
    if (hip4 < 260) hip4 = hip4 + 1;
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  for (; fem3 > 300; fem3 = fem3 - 10) {
    if (hip1 < 400) hip1 = hip1 + 1;
    if (hip2 < 270) hip2 = hip2 + 1;
    if (hip4 < 270) hip4 = hip4 + 1;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(12, 0, hip4);
    delay(delayTime);
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  // *********************** End turning left ****************************************
}

void back() {
  // ********************** Walking Backwards from Neutral standing *******************
  if (legLevel < 5 || togglePosture != 1) { // Check if front legs are out (legLevel<5) or if not standing (toggle not equal to 1)
    stand ();
  }
  // ******** Walking Backwards: first shift towards leg 2: Positions of neutral standing

  for (; tib4 < 475; tib4 = tib4 + 2) {
    if (hip1 < 500) hip1 = hip1 + 2;
    if (tib2 > 325) tib2 = tib2 - 2;
    if (hip2 < 350) hip2 = hip2 + 1;
    if (fem2 < 410) fem2 = fem2 + 1;
    if (hip3 > 375) hip3 = hip3 - 1;
    if (fem4 < 425) fem4 = fem4 + 1;
    pwm.setPWM(14, 0, tib4);
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(6, 0, tib2);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(13, 0, fem4);
    delay(delayTime);
  }

  // ************** Walking Backward: Second moving leg4 backward **************************
  for (; fem4 > 325; fem4 = fem4 - 2) {
    if (tib4 > 275) tib4 = tib4 - 5;
    if (fem2 > 290) fem2 = fem2 - 1;
    if (fem3 > 280) fem3 = fem3 - 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(14, 0, tib4);
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(9, 0, fem3);
    delay (delayTime);
  }
  for (; hip4 < 420; hip4 += 2) {
    pwm.setPWM(12, 0, hip4);
    delay (delayTime);
  }
  for (; fem4 < 400; fem4 = fem4 + 1) {
    if (tib4 < 375) tib4 = tib4 + 2;
    if (fem1 < 300) fem1 = fem1 + 1;
    if (fem3 < 300) fem3 = fem3 + 1;
    pwm.setPWM(13, 0, fem4);
    pwm.setPWM(14, 0, tib4);
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(9, 0, fem3);
    delay (delayTime);
  }

  // ************** Walking backward: Third - moving leg 3 backward ***************************
  for (; fem3 < 350; fem3 = fem3 + 1) {
    if (tib3 < 425) tib3 = tib3 + 2;
    pwm.setPWM(9, 0, fem3);
    pwm.setPWM(10, 0, tib3);
    delay (delayTime);
  }
  for (; hip3 < 495; hip3 = hip3 + 2) {
    if (tib3 > 350) tib3 = tib3 + 2;
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(10, 0, tib3);
    delay (delayTime);
  }
  for (; tib3 > 250; tib3 = tib3 - 2) {
    pwm.setPWM(10, 0, tib3);
    delay (delayTime);
  }
  for (; tib3 > 175; tib3 = tib3 - 2) {
    if (fem3 > 275) fem3 = fem3 - 2;
    pwm.setPWM(10, 0, tib3);
    pwm.setPWM(9, 0, fem3);
    delay (delayTime);
  }

  // ******* Walking backwards: Forth - shift towards leg3 and long step forward with leg1 *******
  for (; hip4 > 275; hip4 = hip4 - 2) {
    if (tib3 < 330) tib3 = tib3 + 2;
    if (tib1 > 200) tib1 = tib1 - 2;
    if (hip2 < 425) hip2 = hip2 + 2;
    if (fem3 < 300) fem3 = fem3 + 1;
    pwm.setPWM(12, 0, hip4);
    pwm.setPWM(10, 0, tib3);
    pwm.setPWM(2, 0, tib1);
    pwm.setPWM(4, 0, hip2);
    pwm.setPWM(9, 0, fem3);
    delay (delayTime);
  }
  for (; fem1 < 375; fem1 = fem1 + 1) {
    if (tib1 < 375) tib1 = tib1 + 3;
    if (hip4 > 250) hip4 = hip4 - 1;
    if (fem4 < 425) fem4 = fem4 + 1;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(2, 0, tib1);
    pwm.setPWM(12, 0, hip4);
    pwm.setPWM(13, 0, fem4);
    delay (delayTime);
  }

  for (; hip1 > 305; hip1 = hip1 - 2) {
    pwm.setPWM(0, 0, hip1);
    delay (delayTime);
  }
  for (; fem1 > 300; fem1 = fem1 - 1) {
    if (tib1 > 330) tib1 = tib1 - 1;
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(2, 0, tib1);
    delay (delayTime);
  }



  // ******** Walking backwards: Fifth - moving leg2 posteriorly ******************************
  for (; fem2 > 250; fem2 = fem2 - 1) {
    if (tib2 > 250) tib2 = tib2 - 1;
    if (fem1 > 275) fem1 = fem1 - 1;
    pwm.setPWM(5, 0, fem2);
    pwm.setPWM(6, 0, tib2);
    pwm.setPWM(1, 0, fem1);
    delay (delayTime);
  }
  for (; hip2 > 270; hip2 = hip2 - 2) {
    pwm.setPWM(4, 0, hip2);
    delay (delayTime);
  }
  for (; tib2 < 325; tib2 = tib2 + 1) {
    pwm.setPWM(6, 0, tib2);
    delay (delayTime);
  }
  for (; fem2 < 400; fem2 = fem2 + 1) {
    if (hip1 < 400) hip1 = hip1 + 2;
    if (fem1 < 300) fem1 = fem1 + 1;
    if (tib2 < 400) tib2 = tib2 + 1;
    if (hip3 > 420) hip3 = hip3 - 1;
    if (hip4 < 270) hip4 = hip4 + 1;
    if (fem4 > 400) fem4 = fem4 - 1;
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1);
    pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
    pwm.setPWM(8, 0, hip3);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4);
    delay (delayTime);
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  // ************* End of Walking Backwards ****************************************
}

void skimPosture() {
  // *******This puts the robot in a posture for skimming with all legs out laterally ***********
  if (legLevel < 5) {
    for (; tib1 > 100; tib1 = tib1 - 5) { // Lowering legs in case something is being gripped
      if (tib4 < 580) tib4 = tib4 + 5;
      if (fem1 > 330) fem1 = fem1 - 3;
      if (fem4 < 360) fem4 = fem4 + 3;
      pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
      pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
      delay(10);
    }
    {
      for (; legDist > 0; legDist = legDist - 1) {
        // code for brining front apart in case something was being gripped
        int Lt = map(legDist, 0, 20, LtApartLimit, LtTogetherLimit);
        int Rt = map (legDist, 0, 20, RtApartLimit, RtTogetherLimit);
        pwm.setPWM(0, 0, Lt); //hip1_545 anterior, 400 for neutral standing
        pwm.setPWM(12, 0, Rt); //hip4_150 anterior, 270 for neutral standing
      }
    } // now in the next section all 4 legs should retract
  }
  if (togglePosture != 0) {
    hip1 = 350, fem1 = 490, tib1 = 500; // flexing all the legs, with hips lateral
    hip2 = 340, fem2 = 200, tib2 = 230;
    hip3 = 350, fem3 = 490, tib3 = 500;
    hip4 = 350, fem4 = 200, tib4 = 200;
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
    pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
    delay (50);
    legLevel = 5;
  }

  hip1 = 350, fem1 = 360, tib1 = 200; // legs out for Skimming posture
  hip2 = 340, fem2 = 360, tib2 = 560; // hip2 = 350
  hip3 = 350, fem3 = 375, tib3 = 200; //hip3 = 350
  hip4 = 350, fem4 = 325, tib4 = 490;
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
    pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
    pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
  delay (50);
  togglePosture = 0; // zero because it is now in skimming stance
  legLevel = 5; // not ready to grip
  digitalWrite(LED_PIN, HIGH); // on standby
  Serial.println("end Skim posture");
}

void pause() {
  // ********** This retracts all the legs and the robot does nothing *****************
  hip1 = 400, fem1 = 500, tib1 = 500; // flexing all the legs, with hips neutral
  hip2 = 270, fem2 = 200, tib2 = 230;
  hip3 = 420, fem3 = 525, tib3 = 500;
  hip4 = 270, fem4 = 200, tib4 = 200;
  pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
  pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
  pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
  pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
  delay (delayTime + 1);
  togglePosture = 3; // because neither standing or skim stance
  legLevel = 5; // front legs not ready to grip
  digitalWrite(LED_PIN, HIGH); // on standby
}


void gripPosture() {
  // this posture puts the back legs out – ready to skim; and the front legs retracted – ready to grip
  if (togglePosture != 2) {
    hip1 = 450; fem1 = 490; tib1 = 500; // flexed in neutral
    hip4 = 220; fem4 = 200; tib4 = 180; // flexed in neutral
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
    togglePosture = 2; // because now in gripper posture
    legDist = 0; // for the position of the front legs out diagonally (neutral)
    legLevel = 5; // front legs flexed
  }
  // and whether already in gripping mode or not, put the back legs ready
  hip2 = 340; fem2 = 360, tib2 = 560; // out to side ready for skimming
  hip3 = 350; fem3 = 375, tib3 = 200; // out to side ready for skimming
  pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
  pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
  digitalWrite(LED_PIN, HIGH); // on standby
} // end of gripPosture

void gripTogether() {
  // ********* This moves the front legs towards the anterior, there are 21 increments 0-20, with 20 being both legs straight anterior
  if (togglePosture == 2) { // Only works in gripper mode, so must be 2
    legDist = legDist + 1;
    if (legDist > 20) legDist = 20;
    int Lt = map(legDist, 0, 20, LtApartLimit, LtTogetherLimit);
    int Rt = map (legDist, 0, 20, RtApartLimit, RtTogetherLimit);
    pwm.setPWM(0, 0, Lt); //hip1_545 anterior, 400 for neutral standing
    pwm.setPWM(12, 0, Rt); //hip4_150 anterior, 270 for neutral standing
  }
  digitalWrite(LED_PIN, HIGH); // on standby
}

void gripApart()
{
  // ********* This moves the front legs out to the sides, there are 21 increments 0-20, with 0 being each leg diagonal (neutral) to the rest of the body
  if (togglePosture == 2) { // Only works in gripper mode
    legDist = legDist - 2;
    if (legDist < 0) legDist = 0;
    int Lt = map(legDist, 0, 20, LtApartLimit, LtTogetherLimit);
    int Rt = map (legDist, 0, 20, RtApartLimit, RtTogetherLimit);
    pwm.setPWM(0, 0, Lt); //hip1_545 anterior, 400 for neutral standing
    pwm.setPWM(12, 0, Rt); //hip4_150 anterior, 270 for neutral standing
  }
  digitalWrite(LED_PIN, HIGH); // on standby
}

void gripUp() {
  // *************** This moves front gripping legs up, there are 5 increments, where 5 is fully retracted, 4 is the highest, 0 is the lowest
  if (togglePosture == 2) { // Only works in gripper mode
    legLevel = legLevel + 1;
    if (legLevel > 5) legLevel = 5;
    switch (legLevel) {
      case 5: // flexed
        {
          fem1 = 490, tib1 = 500;
          fem4 = 200, tib4 = 180;
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 4: // highest – femurs still flexed but tibias level
        {
          fem1 = 490, tib1 = 300; //tib1 is more extended with lower values
          fem4 = 200, tib4 = 400; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 3: // 2nd highest position
        {
          fem1 = 446, tib1 = 270; //tib1 is more extended with lower values
          fem4 = 240, tib4 = 450; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 2: // mid position
        fem1 = 400, tib1 = 205; //tib1 is more extended with lower values
        fem4 = 280, tib4 = 495; //tib4 is more extended with higher values
        pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
        pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
        delay(10);
        {
          break;
        }
      case 1: // Second lowest position
        {
          fem1 = 360, tib1 = 160; //tib1 is more extended with lower values
          fem4 = 320, tib4 = 550; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 0: // Lowest position for the front legs
        {
          fem1 = 315, tib1 = 95; //tib1 is more extended with lower values
          fem4 = 360, tib4 = 585; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
    }
  }
  digitalWrite(LED_PIN, HIGH); // on standby
}

void gripDown() {
  /* ************** This moves front gripping legs down, there are 4 possible increments, if the front legs are retracted the front tibias will extend so the legs are in the highest position, case 0 is the lowest
  */
  if (togglePosture == 2) { // Only works in gripper mode
    legLevel = legLevel - 1;
    if (legLevel < 0) legLevel = 0;
    switch (legLevel) {
      case 5: // flexed
        {
          fem1 = 490, tib1 = 500;
          fem4 = 200, tib4 = 180;
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 4: // highest – femurs still flexed but tibias level
        {
          fem1 = 490, tib1 = 300; //tib1 is more extended with lower values
          fem4 = 200, tib4 = 400; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 3: // 2nd highest position
        {
          fem1 = 446, tib1 = 280; //tib1 is more extended with lower values
          fem4 = 240, tib4 = 440; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 2: // mid position
        fem1 = 400, tib1 = 205; //tib1 is more extended with lower values
        fem4 = 280, tib4 = 495; //tib4 is more extended with higher values
        pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
        pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
        delay(10);
        {
          break;
        }
      case 1: // Second lowest position
        {
          fem1 = 360, tib1 = 160; //tib1 is more extended with lower values
          fem4 = 320, tib4 = 550; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
      case 0: // Lowest position for the front legs
        {
          fem1 = 315, tib1 = 95; //tib1 is more extended with lower values
          fem4 = 360, tib4 = 585; //tib4 is more extended with higher values
          pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
          pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
          delay(10);
          break;
        }
    }
  }
  digitalWrite(LED_PIN, HIGH); // on standby
}


void skimLegsUp() {
  // The movement of the legs up when skimming is repetative so here is a function to do it
  fem2 = 330;
  fem3 = 390;
  pwm.setPWM(5, 0, fem2);
  pwm.setPWM(9, 0, fem3);
  if (togglePosture == 0) {
    fem1 = 375; pwm.setPWM(1, 0, fem1);
    fem4 = 300; pwm.setPWM(13, 0, fem4);
  }
  digitalWrite(LED_PIN, HIGH); // on standby
}

void skimLegsDown() {
  // The movement of the legs down when skimming is repetative so here is a function to do it
  fem2 = 400, tib2 = 560, fem3 = 330, tib3 = 170; //femurs down for feet to touch
  pwm.setPWM(5, 0, fem2); pwm.setPWM(6, 0, tib2);
  pwm.setPWM(9, 0, fem3); pwm.setPWM(10, 0, tib3);
  if (togglePosture == 0) {
    fem1 = 310, tib1 = 160; pwm.setPWM(1, 0, fem1); pwm.setPWM(2, 0, tib1);
    fem4 = 370, tib4 = 540; pwm.setPWM(13, 0, fem4); pwm.setPWM(14, 0, tib4);
  }
  digitalWrite(LED_PIN, HIGH); // on standby
}


void skimForward() {
  // ****** Skim forward on the ground with either all four legs, or just the back two if in gripper posture
  if (togglePosture == 1 || togglePosture == 3) { //checking if not in a skimmer posture, which would be either 0 for skimmer or 2 for grip
    skimPosture();
  }

  skimLegsUp();
  hip2 = 425, hip3 = 275; // hips forward with femurs up
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(8, 0, hip3);
  if (togglePosture == 0) {
    hip1 = 425; pwm.setPWM(0, 0, hip1);
    hip4 = 275; pwm.setPWM(12, 0, hip4);
  }
  delay(100);

  skimLegsDown();
  delay(100); //30


  hip2 = 245;  // STROKE: hips backward, with femurs down just a little
  hip3 = 475;
  pwm.setPWM(4, 0, hip2); pwm.setPWM(5, 0, fem2);
  pwm.setPWM(8, 0, hip3); pwm.setPWM(9, 0, fem3);
  if (togglePosture == 0) {
    hip1 = 255;
    hip4 = 475;
    pwm.setPWM(0, 0, hip1); pwm.setPWM(1, 0, fem1);
    pwm.setPWM(12, 0, hip4); pwm.setPWM(13, 0, fem4);
  }
  delay(rowTime);

  skimLegsUp(); // bring legs up before returning to skimPosition
  delay(10);

  if (togglePosture == 2) {
    gripPosture(); // to reset the back legs to gripPosture positions
  } else {
    skimPosture(); // to reset all the legs to skimPosture positions
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  Serial.println("go forward!");
}


void skimBack() {
  // ******* skim backwards on the ground with all 4 legs or just the back 2 if in gripper posture
  if (togglePosture == 1 || togglePosture == 3) { //checking if not in a skimmer posture, which would be either 0 or 2
    skimPosture();
  }

  skimLegsUp();
  hip2 = 275, hip3 = 440; // hips backward and femurs up
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(8, 0, hip3);
  if (togglePosture == 0) {
    hip1 = 225, hip4 = 475;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(12, 0, hip4);
  }
  delay(100);

  skimLegsDown();
  delay(30);

  hip2 = 450, hip3 = 255; // STROKE: hips forward
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(8, 0, hip3);
  if (togglePosture == 0) {
    hip1 = 465; pwm.setPWM(0, 0, hip1);
    hip4 = 230; pwm.setPWM(12, 0, hip4);
  }
  delay(rowTime);

  skimLegsUp(); // bring legs up before returning to skimPosition
  delay(10);

  if (togglePosture == 2) {
    gripPosture(); // to reset the back legs to gripPosture positions
  } else {
    skimPosture(); // to reset all the legs to skimPosture positions
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  Serial.println("go back!");
}

void skimLeft() {
  // ************ skim on the ground to the left with all 4 legs, or just back two if in gripper posture
  if (togglePosture == 1 || togglePosture == 3) { //checking if not in a skimmer posture, which would be either 0 or 2
    skimPosture();
  }

  skimLegsUp();
  hip2 = 275, hip3 = 290; // left hips backward 35 degrees with femur up
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(8, 0, hip3);
  if (togglePosture == 0) {
    hip1 = 285, hip4 = 265;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(12, 0, hip4);
  }
  delay(100);

  skimLegsDown();
  delay(30);

  hip2 = 450; // STROKE: left hips forward to 50 degrees anterior
  hip3 = 475; // STROKE: right hips backward 50 degrees anterior
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(8, 0, hip3);
  if (togglePosture == 0) {
    hip1 = 465; pwm.setPWM(0, 0, hip1);
    hip4 = 455; pwm.setPWM(12, 0, hip4);
  }
  delay(rowTime);

  skimLegsUp(); // bring legs up before returning to skimPosition
  delay(10);

  if (togglePosture == 2) {
    gripPosture(); // to reset the back legs to gripPosture positions
  } else {
    skimPosture(); // to reset all the legs to skimPosture positions
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  Serial.println("go left!");
}

void skimRight() {
  // ********** skim on the ground with all 4 legs, or just back two if in gripper posutre
  if (togglePosture == 1 || togglePosture == 3) { //checking if not in a skimmer posture, which would be either 0 or 2
    skimPosture();
  }

  skimLegsUp();
  hip2 = 415, hip3 = 440; // left hips forward 35 degrees and femurs up; right hips backward 35 degrees and femurs up
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(8, 0, hip3);
  if (togglePosture == 0) {
    hip1 = 430, hip4 = 415;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(12, 0, hip4);
  }
  delay(100);

  skimLegsDown();
  delay(30);

  hip2 = 245, fem2 = 380; // STROKE: left hips backward to 50 degrees, femurs down a little
  hip3 = 255, fem3 = 360; // STROKE: right hips forward to 50 degrees, femurs down a little
  pwm.setPWM(4, 0, hip2);
  pwm.setPWM(5, 0, fem2);
  pwm.setPWM(8, 0, hip3);
  pwm.setPWM(9, 0, fem3);
  if (togglePosture == 0) {
    hip1 = 255, fem1 = 340;
    hip4 = 230, fem4 = 345;
    pwm.setPWM(0, 0, hip1);
    pwm.setPWM(1, 0, fem1);
    pwm.setPWM(12, 0, hip4);
    pwm.setPWM(13, 0, fem4);
  }
  delay(rowTime);

  skimLegsUp(); // bring legs up before returning to skimPosition
  delay(10);

  if (togglePosture == 2) {
    gripPosture(); // to reset the back legs to gripPosture positions
  } else {
    skimPosture(); // to reset all the legs to skimPosture positions
  }
  digitalWrite(LED_PIN, HIGH); // on standby
  Serial.println("go right!");
}


void setup() {
  Serial.begin(9600); // if needed for troubleshooting

  irrecv.enableIRIn();

  // Starting the PWM servo driver
  pwm.begin();
  pwm.setPWMFreq(60);  // Analog servos run at ~60 Hz updates
  delay(10);

  // setup on the standby LED
  pinMode(LED_PIN, OUTPUT);

  //retract all legs by running pause, will turn on the LED at the end
  pause();
}


void loop()
{
  if (irrecv.decode(&results)) {
    if (results.value != 0xFFFFFF) { //0xFFFFFF is the "repeat" command from the remote, so the robot will ignore it
      digitalWrite(LED_PIN, LOW); // turn off standby LED
      val = results.value;
      irrecv.resume();
      switch (val) { // Which functions to call when button is pressed, buttons were defined before the setup
        case P: pause (); break;
        case Fr: forward(); break;
        case B: back(); break;
        case L: left(); break;
        case R: right(); break;
        case St: stand(); break;
        case Lu: gripUp(); break;
        case Ld: gripDown(); break;
        case lt: gripTogether(); break;
        case Eq: gripPosture(); break;
        case La: gripApart(); break;
        case Sf: skimForward(); break;
        case Sb: skimBack(); break;
        case Sr: skimRight(); break;
        case Sl: skimLeft(); break;
        case S: skimPosture(); break;
        default: break;
      }
    }
  }
  digitalWrite(LED_PIN,HIGH); // turn on the standby LED just in case
}
